// Smets & Wouters model - with indexation
// with BGG financial sector
// with two shocks - 'b' and 'fw'
// includes inflatione expectation and conditional forecast following 'Negro, Giannoni & Schorfheide' (2014)
// Callibrated for NGSffusmodel4wp3_81to08Q3|Y+
// copy van usmodel_hist_dsge_f19_7_71


//VARIABLE & PARAMETER DECLARATION
var b spread labobs robs pinfobs dy dc dinve dw  ewma epinfma  zcapf rkf kf pkf    cf invef yf labf wf rrf mc zcap rk k pk    c inve y lab pinf w r a g qs  ms  spinf sw kpf kp 
re ref n nf fw inflcast10yr pinfstar;   
 
varexo ea efw eg eqs em ew eb epst epinf;  

parameters 
curvw cgy curvp cmaw cmap calfa 
czcap cbeta csadjcost ctou csigma chabb cfc 
cindw cprobw cindp cprobp csigl clandaw 
crpi crdy cry crr 
crhoa crhog crhoqs crhoms crhopinf crhow crhob crhops
ctrend constelab constepinf
cg crk cz comegabar cmeu1 cnkbar1 cnkbar2
cprobsp cfstar cserv csigfw crhofw cefwn
cspread;
                                    //parameters from 'usmodel' not initialized: crdpi ccs cinvs 


//VARIABLES INITIALISATION
// fixed parameters
ctou        =0.025;
clandaw     =1.5;
cg          =0.18;
curvp       =20;
curvw       =10;
cbeta       =0.9995;

// estimated parameters initialisation
crhoa       =0.9127;
crhofw      =0.9847;
cefwn       =0.7429;
crhob       =0.9939;
crhog       =0.9874;
crhoqs      =0.9978;
crhoms      =0.1045;
crhopinf    =0.8124;
crhops      =0.9051;
crhow       =0.5380;
cmap        =0.6941;
cmaw        =0.4458;

csadjcost   =3.5461;
csigma      =1.9528;
chabb       =0.3267;
cprobw      =0.8657;
csigl       =2.2910;
//cprobp      =0.8519;
cprobp      =0.60;
cindw       =0.3840;
cindp       =0.2390;
czcap       =0.4964;
cfc         =1.5934;
crpi        =1.3253;
crr         =0.7373;
cry         =0.0785;
crdy        =0.1756;
constepinf  =0.5986;
constelab   =1.8441; 
ctrend      =0.5704;    
cgy         =0.4489;
calfa       =0.2505;
cspread     =1.8695;
cprobsp     =0.0485;                       //change in cprobsp has significant effect on the estimated values for investment and financial parameters


//financial sector
cfstar      =0.0026;                      //business failure rate (0.03 in BGG). quarterly equivalent of 0.01 is 0.0026
cserv       =0.9728;                      //entrepreneurs srvival rate
csigfw      =sqrt(0.24);                  //sqrt(0.24)
crk         =0.0448;                      //rental rate of capital (0.59 in CMR 2010). treated as exogenous

cz=norminv(cfstar);                                                         //-1.8808;//-2.3263                  //eq 7.30 in SN appendix
comegabar=exp(csigfw*cz - 0.5*(csigfw^2));                                  //0.3459;//0.2837                    //eq 7.31 in SN appendix
cmeu1=((normpdf(cz)/(csigfw*(1-cfstar)))*(1-normcdf(cz-csigfw)-comegabar*(1-cfstar))+normcdf(cz-csigfw));
cnkbar1=comegabar*(1-cfstar);
cnkbar2=normcdf(cz-csigfw);

//MODEL BLOCK
model(linear); 
# cpie=1+constepinf/100;
# cgamma=1+ctrend/100 ;
# constebeta=(1/cbeta-1)*100;
# clandap=cfc;
# cbetabar=cbeta*cgamma^(-csigma);
# cr=cpie/(cbeta*cgamma^(-csigma));
# cre=(crk+(1-ctou))*cpie;//1.0392;//1.0249//                       //eq 7.15 in SN appendix
# cw = (calfa^calfa*(1-calfa)^(1-calfa)/(clandap*crk^calfa))^(1/(1-calfa));
# cikbar=(1-(1-ctou)/cgamma);
# cik=(1-(1-ctou)/cgamma)*cgamma;
# clk=((1-calfa)/calfa)*(crk/cw);
# cky=cfc*(clk)^(calfa-1);
# ciy=cik*cky;
# ccy=1-cg-cik*cky;
# crkky=crk*cky;
# cwhlc=(1/clandaw)*(1-calfa)/calfa*crk*cky/ccy;
# cwly=1-crk*cky;
# conster=(cr-1)*100;

# cmeu=(1-(cre/cr)^(-1))/cmeu1;
# cnkbar=1-(cnkbar1+(1-cmeu)*cnkbar2)*(cre/cr);//0.7437;//0.7137
# ckbarn=1/cnkbar;

# ckappa = ((1-cbetabar*cgamma*cprobp)*(1-cprobp))/((1+cbetabar*cgamma*cindp)*cprobp*((cfc-1)*curvp+1));

// flexible economy
          //financial sector
          ref(+1)-rrf = b + cprobsp*(pkf + kpf - nf) + fw;
          ref = (crk/(crk+(1-ctou)))*rkf +  ((1-ctou)/(crk+(1-ctou)))*pkf - pkf(-1);
          nf = (cserv*cre*ckbarn/cpie)*(1-cmeu*normcdf(cz-csigfw))*ref - (cserv/cbeta)*(ckbarn-1)*rrf(-1) + (cserv*cre*ckbarn/cpie)*(1-(cpie/(cbeta*cre))-cmeu*normcdf(cz-csigfw))*(pkf(-1)+kpf(-1)) + (cserv/cbeta)*nf(-1);                  //linearized as in SN (2012) appendix

          //others
	      0*(1-calfa)*a + 1*a =  calfa*rkf+(1-calfa)*(wf)  ;
	      zcapf =  (1/(czcap/(1-czcap)))* rkf  ;
	      rkf =  (wf)+labf-kf ;
	      kf =  kpf(-1)+zcapf ;
	      invef = (1/(1+cbetabar*cgamma))* (  invef(-1) + cbetabar*cgamma*invef(1)+(1/(cgamma^2*csadjcost))*pkf ) +qs ;
	      cf = (chabb/cgamma)/(1+chabb/cgamma)*cf(-1) + (1/(1+chabb/cgamma))*cf(+1) +((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(labf-labf(+1)) - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(rrf+b);
	      yf = ccy*cf+ciy*invef+g  +  crkky*zcapf ;
	      yf = cfc*( calfa*kf+(1-calfa)*labf +a );
	      wf = csigl*labf 	+(1/(1-chabb/cgamma))*cf - (chabb/cgamma)/(1-chabb/cgamma)*cf(-1) ;
	      kpf =  (1-cikbar)*kpf(-1)+(cikbar)*invef + (cikbar)*(cgamma^2*csadjcost)*qs ;

// sticky price - wage economy
          //financial sector
          re(+1)-r = b + cprobsp*(pk + kp - n) + fw;
          re - pinf = (crk/(crk+(1-ctou)))*rk +  ((1-ctou)/(crk+(1-ctou)))*pk - pk(-1);
          n = (cserv*cre*ckbarn/cpie)*(1-cmeu*normcdf(cz-csigfw))*(re-pinf) - (cserv/cbeta)*(ckbarn-1)*(r(-1)-pinf) + (cserv*cre*ckbarn/cpie)*(1-(cpie/(cbeta*cre))-cmeu*normcdf(cz-csigfw))*(pk(-1)+kp(-1)) + (cserv/cbeta)*n(-1) ;          //linearized as in SN (2012) appendix

          //others
	      mc =  calfa*rk+(1-calfa)*(w) - 1*a - 0*(1-calfa)*a ;                  //varify if it is 'rk' or 'k' ??? in the SW paper in eq 9 it is 'k' not 'rk' which is rental rate of capital.
	      zcap =  (1/(czcap/(1-czcap)))* rk ;
	      rk =  w+lab-k ;
	      k =  kp(-1)+zcap ;
	      inve = (1/(1+cbetabar*cgamma))* (  inve(-1) + cbetabar*cgamma*inve(1)+(1/(cgamma^2*csadjcost))*pk ) +qs ;
	      c = (chabb/cgamma)/(1+chabb/cgamma)*c(-1) + (1/(1+chabb/cgamma))*c(+1) +((csigma-1)*cwhlc/(csigma*(1+chabb/cgamma)))*(lab-lab(+1)) - (1-chabb/cgamma)/(csigma*(1+chabb/cgamma))*(r-pinf(+1)+b);
	      y = ccy*c+ciy*inve+g  +  1*crkky*zcap ;
	      y = cfc*( calfa*k+(1-calfa)*lab +a );
	      pinf =  (1/(1+cbetabar*cgamma*cindp)) * ( cbetabar*cgamma*pinf(1) +cindp*pinf(-1) 
               +((1-cprobp)*(1-cbetabar*cgamma*cprobp)/cprobp)/((cfc-1)*curvp+1)*(mc)  )  + spinf ; 
	      w =  (1/(1+cbetabar*cgamma))*w(-1)
               +(cbetabar*cgamma/(1+cbetabar*cgamma))*w(1)
               +(cindw/(1+cbetabar*cgamma))*pinf(-1)
               -(1+cbetabar*cgamma*cindw)/(1+cbetabar*cgamma)*pinf
               +(cbetabar*cgamma)/(1+cbetabar*cgamma)*pinf(1)
               +(1-cprobw)*(1-cbetabar*cgamma*cprobw)/((1+cbetabar*cgamma)*cprobw)*(1/((clandaw-1)*curvw+1))*
               (csigl*lab + (1/(1-chabb/cgamma))*c - ((chabb/cgamma)/(1-chabb/cgamma))*c(-1) -w) 
               + 1*sw ;
	      r =  crpi*(1-crr)*(pinf-pinfstar)
               +cry*(1-crr)*(y-yf)     
               +crdy*(y-yf-y(-1)+yf(-1))
               +crr*r(-1)
               +ms  ;
	      a = crhoa*a(-1)  + ea;
	      fw = crhofw*fw(-1) + efw + cefwn*efw(-1) + cefwn^2*efw(-2) + cefwn^3*efw(-3) + cefwn^4*efw(-4) + cefwn^5*efw(-5) + cefwn^6*efw(-6) + cefwn^7*efw(-7) + cefwn^8*efw(-8);
	      b = crhob*b(-1) + eb;
	      g = crhog*(g(-1)) + eg + cgy*ea;
	      qs = crhoqs*qs(-1) + eqs;
	      ms = crhoms*ms(-1) + em;
	      spinf = crhopinf*spinf(-1) + epinfma - cmap*epinfma(-1);
	          epinfma=epinf;
	      sw = crhow*sw(-1) + ewma - cmaw*ewma(-1) ;
	          ewma=ew; 
	      pinfstar = crhops*pinfstar(-1) + epst;
          kp =  (1-cikbar)*kp(-1)+cikbar*inve + cikbar*cgamma^2*csadjcost*qs ;

// measurment equations
          dy=y-y(-1)+ctrend;
          dc=c-c(-1)+ctrend;
          dinve=inve-inve(-1)+ctrend;
          dw=w-w(-1)+ctrend;
          pinfobs = 1*(pinf) + constepinf;
          robs =    1*(r) + conster;
          labobs = lab + constelab;
          spread = cspread+re(+1)-r;
          //equation for inflation forecast
          inflcast10yr = constepinf + 1/40*(
                @#for k in 1:40
                    + pinf(@{k})
                @#endfor
          );
end; 


//SHOCKS INITIALISATION
shocks;
var ea;
stderr 0.3627;
var efw;
stderr 0.0523;
var eb;
stderr 0.0288;
var eg;
stderr 0.4441;
var eqs;
stderr 0.3851;
var em;
stderr 0.1947;
var epinf;
stderr 0.1141;
var epst;
stderr 0.1678;
var ew;
stderr 0.3222;
end;


//PRIORS DECLARATION
estimated_params;
// PARAM NAME, INITVAL, LB, UB, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF
stderr ea,0.3572,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr efw,0.0528,INV_GAMMA_PDF,0.05,4;
stderr eb,0.0456,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr eg,0.4509,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr eqs,0.2465,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr em,0.1655,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr epinf,0.0956,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr epst,1.0029,0.01,3,INV_GAMMA_PDF,0.1,2;
stderr ew,0.3545,0.01,3,INV_GAMMA_PDF,0.1,2;

crhoa,.9455 ,.01,.9999,BETA_PDF,0.5,0.20;
crhofw,0.9954,BETA_PDF,0.75,0.15;
cefwn,0.7529,INV_GAMMA_PDF,1,2;
crhob,.9900,.001,.9999,BETA_PDF,0.5,0.20;
crhog,.9873,.01,.9999,BETA_PDF,0.5,0.20;
crhoqs,.9842,.01,.9999,BETA_PDF,0.5,0.20;
crhoms,.0816,.01,.9999,BETA_PDF,0.5,0.20;
crhopinf,.9343,.01,.9999,BETA_PDF,0.5,0.20;
crhops,.5920,.001,.9999,BETA_PDF,0.5,0.20;
crhow,.9025,.001,.9999,BETA_PDF,0.5,0.20;
cmap,.6461,0.01,.9999,BETA_PDF,0.5,0.2;
cmaw,.5186,0.01,.9999,BETA_PDF,0.5,0.2;
cgy,0.4268,0.01,2.0,BETA_PDF,0.5,0.2;

csadjcost,4.7392,2,15,NORMAL_PDF,4,1.5;
csigma,2.88,0.25,6,NORMAL_PDF,1.50,0.375;
chabb,0.3370,0.001,0.99,BETA_PDF,0.7,0.1;
cprobw,0.6785,0.3,0.95,BETA_PDF,0.5,0.1;
csigl,1.2011,0.25,10,NORMAL_PDF,2,0.75;
//cprobp,0.6713,0.5,0.95,BETA_PDF,0.5,0.10;
cindw,0.5130,0.01,0.99,BETA_PDF,0.5,0.15;
cindp,0.5567,0.01,0.99,BETA_PDF,0.5,0.15;
czcap,0.8478,0.01,1,BETA_PDF,0.5,0.15;
cfc,1.5657,1.0,3,NORMAL_PDF,1.25,0.125;
crpi,1.3369,1.01,3,NORMAL_PDF,1.5,0.25;
crr,0.7604,0.5,0.975,BETA_PDF,0.75,0.10;
cry,0.0001,-0.1,0.5,NORMAL_PDF,0.125,0.05;
crdy,0.1888,0.001,0.5,NORMAL_PDF,0.125,0.05;
constepinf,0.4556,0.1,2.0,GAMMA_PDF,0.625,0.1;//20;
constelab,0.8974,-10.0,10.0,NORMAL_PDF,0.0,2.0;
ctrend,0.5013,0.1,0.8,NORMAL_PDF,0.4,0.10;
calfa,0.2317,0.01,1.0,NORMAL_PDF,0.3,0.05;

cspread,1.8682,GAMMA_PDF,2.0,0.10;
cprobsp,0.0474,BETA_PDF,0.05,0.005;
end;


//DATA VARIABLES
varobs dy dc dinve labobs pinfobs dw robs spread inflcast10yr;


//ESTIAMTION & SIMULATIONS
// first_obs=41 is 1965Q1 (upto 2008Q3 includes nobs=175)
//estimation(optim=('MaxIter',300),datafile=data_spread_55Q1to13Q2,mode_compute=1,first_obs=37,nobs=175,smoother,presample=4,lik_init=2,prefilter=0,mh_replic=20000,mh_nblocks=2,mh_jscale=0.30,mh_drop=0.2,forecast=19,nodisplay) dy dc dinve labobs pinfobs dw robs mc spread Finfl Minfl;
// first_obs=1 is 1981Q1 (upto 2008Q3 includes nobs=111)
//estimation(optim=('MaxIter',200),datafile=data_spread13_inflcast,mode_compute=1,first_obs=1,nobs=111,smoother,presample=4,lik_init=2,prefilter=0,mh_replic=20000,mh_nblocks=2,mh_jscale=0.25,mh_drop=0.2,forecast=19,nodisplay);// dy dc dinve labobs pinfobs dw robs mc spread Finfl;
estimation(optim=('MaxIter',200),datafile=data_spread13_inflcast,mode_file=NGSusmodelffnews_4wp3_mode,mode_compute=0,first_obs=1,nobs=111,smoother,presample=4,lik_init=2,prefilter=0,load_mh_file,mh_replic=0,mh_nblocks=2,mh_jscale=0.25,mh_drop=0.2,forecast=19,nodisplay);// dy dc dinve labobs pinfobs dw robs mc spread pfp;// mcs mcf pinfs mcs1 mcs2;// pinff;// Finfl;



conditional_forecast_paths;
var robs;
periods  1;// 2;//:19;
values   0.13;// 0.05;
var spread;
periods  1;//   2;//    3    4   5   6   7   8   9   10  11  12  13  14  15  16  17  18  19;
values   1.4;// 1.37;// 1.17 .79 .72 .64 .67 .75 .76 .66 .66 .76 .80 .79 .82 .81 .72 .72 .71;
//var mc;
//periods   1     2     3     4     5     6     7     8     9     10    11    12    13     14    15    16     17    18     19;
//values   -3.45 -7.86 -6.67 -7.35 -8.15 -9.18 -8.25 -8.37 -9.09 -7.83 -9.14 -9.23 -10.84 -9.15 -9.57 -10.51 -8.05 -10.40 -10.25;
end;


//conditional_forecast(parameter_set=calibration, controlled_varexo=(em, efw),periods=19);
conditional_forecast(parameter_set=posterior_mean, controlled_varexo=(em, efw),periods=19);

//plot_conditional_forecast(periods=19) dy pinfobs mc robs spread;// Finfl Minfl;


//stoch_simul(conditional_variance_decomposition = [1 10 40],irf=19) dy dc dinve labobs pinfobs dw robs mc;
stoch_simul(irf=19);// dy dc dinve labobs pinfobs dw robs mc;